/*
 * Copyright 2008 Oliver Zoran
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package gwt.canvas.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.Image;

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/#the-canvas
 * @see http://www.w3.org/html/wg/html5/#the-canvas
 * @see http://canvex.lazyilluminati.com/tests/tests/results.html
 */
public class CanvasImpl {

	@SuppressWarnings("unused")
	private JavaScriptObject context;

	protected Element element;

	protected String backgroundColor;

	public void init(Canvas canvas) {
		this.element = canvas.getElement();
		init();
	}

	protected native void init() /*-{
		this.@gwt.canvas.client.CanvasImpl::context = this.@gwt.canvas.client.CanvasImpl::element.getContext("2d");
	}-*/;

	public void setHeight(int height) {
		DOM.setElementAttribute(element, "height", String.valueOf(height));
	}

	public void setWidth(int width) {
		DOM.setElementAttribute(element, "width", String.valueOf(width));
	}

	public void setBackgroundColor(String color) {
		backgroundColor = color;
		DOM.setStyleAttribute(element, "background", color);
	}

	/////////////////////////////////////////////////////////////////
	// CANVAS STATE METHODS
	/////////////////////////////////////////////////////////////////

	public native void restore() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.restore();
	}-*/;

	public native void save() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.save();
	}-*/;

	public native void rotate(float angle) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.rotate(angle);
	}-*/;

	public native void scale(float x, float y) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.scale(x, y);
	}-*/;

	public native void translate(float x, float y) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.translate(x, y);
	}-*/;

	public native void transform(float m11, float m12, float m21, float m22, float dx, float dy) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.transform(m11, m12, m21, m22, dx, dy);
	}-*/;

	public native void setTransform(float m11, float m12, float m21, float m22, float dx, float dy) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.setTransform(m11, m12, m21, m22, dx, dy);
	}-*/;

	/////////////////////////////////////////////////////////////////
	// WORKING WITH PATHS
	/////////////////////////////////////////////////////////////////

	public native void arc(float x, float y, float radius, float startAngle, float endAngle, boolean anticlockwise) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.arc(x, y, radius, startAngle, endAngle, anticlockwise);
	}-*/;

	public native void cubicCurveTo(float cp1x, float cp1y, float cp2x, float cp2y, float x, float y) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.bezierCurveTo(cp1x, cp1y, cp2x, cp2y, x, y);
	}-*/;

	public native void quadraticCurveTo(float cpx, float cpy, float x, float y) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.quadraticCurveTo(cpx, cpy, x, y);
	}-*/;

	public native void beginPath() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.beginPath();
	}-*/;

	public native void closePath() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.closePath();
	}-*/;

	public native void moveTo(float x, float y) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.moveTo(x, y);
	}-*/;

	public native void lineTo(float x, float y) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.lineTo(x, y);
	}-*/;

	public native void rect(float x, float y, float w, float h) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.rect(x, y, w, h);
	}-*/;

	/////////////////////////////////////////////////////////////////
	// STROKING AND FILLING
	/////////////////////////////////////////////////////////////////

	public native void clear() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.clearRect(-1e4, -1e4, 2e4, 2e4);
	}-*/;

	public native void fill() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.fill();
	}-*/;

	public native void stroke() /*-{
		this.@gwt.canvas.client.CanvasImpl::context.stroke();
	}-*/;

	public native void clearRect(float x, float y, float w, float h) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.clearRect(x, y, w, h);
	}-*/;

	public native void fillRect(float x, float y, float w, float h) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.fillRect(x, y, w, h);
	}-*/;

	public native void strokeRect(float x, float y, float w, float h) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.strokeRect(x, y, w, h);
	}-*/;

	/////////////////////////////////////////////////////////////////
	// GRADIENT AND PATTERN STYLES
	/////////////////////////////////////////////////////////////////

	public native void createLinearGradient(float x0, float y0, float x1, float y1) /*-{
		// TODO implement and test
	}-*/;

	public native void createPattern(Image image, String repetition) /*-{
		// TODO implement and test
	}-*/;

	public native void createRadialGradient(float x0, float y0, float r0, float x1, float y1, float r1) /*-{
		// TODO implement and test
	}-*/;

	/////////////////////////////////////////////////////////////////
	// DRAWING IMAGES
	/////////////////////////////////////////////////////////////////

	public native void drawImage(Image image, float x, float y) /*-{
		// TODO implement and test
	}-*/;

	public native void drawImage(Image image, float x, float y, float width, float height) /*-{
		// TODO implement and test
	}-*/;

	public native void drawImage(Image image, float sx, float sy, float swidth, float sheight,
			float dx, float dy, float dwidth, float dheight)
	/*-{
		// TODO implement and test
	}-*/;

	/////////////////////////////////////////////////////////////////
	// SETTERS AND GETTERS
	/////////////////////////////////////////////////////////////////

	public native void setGlobalAlpha(float globalAlpha) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.globalAlpha = globalAlpha;
	}-*/;

	public native float getGlobalAlpha() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.globalAlpha;
	}-*/;

	public native void setGlobalCompositeOperation(String globalCompositeOperation) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.globalCompositeOperation = globalCompositeOperation;
	}-*/;

	public native String getGlobalCompositeOperation() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.globalCompositeOperation;
	}-*/;

	public native void setStrokeStyle(String strokeStyle) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.strokeStyle = strokeStyle;
	}-*/;

	public native String getStrokeStyle() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.strokeStyle;
	}-*/;

	public native void setFillStyle(String fillStyle) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.fillStyle = fillStyle;
	}-*/;

	public native String getFillStyle() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.fillStyle;
	}-*/;

	public native void setLineWidth(float lineWidth) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.lineWidth = lineWidth;
	}-*/;

	public native float getLineWidth() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.lineWidth;
	}-*/;

	public native void setLineCap(String lineCap) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.lineCap = lineCap;
	}-*/;

	public native String getLineCap() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.lineCap;
	}-*/;

	public native void setLineJoin(String lineJoin) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.lineJoin = lineJoin;
	}-*/;

	public native String getLineJoin() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.lineJoin;
	}-*/;

	public native void setMiterLimit(float miterLimit) /*-{
		this.@gwt.canvas.client.CanvasImpl::context.miterLimit = miterLimit;
	}-*/;

	public native float getMiterLimit() /*-{
		return this.@gwt.canvas.client.CanvasImpl::context.miterLimit;
	}-*/;
}
