/*
 * Copyright 2008 Oliver Zoran
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package gwt.canvas.client;

public class CanvasImplSafariOpera extends CanvasImpl {

	private boolean pathIsOpen = false;

	public void beginPath() {
		super.beginPath();
		pathIsOpen = true;
	}

	public void closePath() {
		super.closePath();
		pathIsOpen = false;
	}

	public void stroke() {
		super.stroke();
		pathIsOpen = false;
	}

	public void fill() {
		super.fill();
		pathIsOpen = false;
	}

	public void clear() {
		if (pathIsOpen) {
			super.beginPath();
		}
		super.clear();
	}

	public void clearRect(float x, float y, float w, float h) {
		if (pathIsOpen) {
			super.beginPath();
		}
		super.clearRect(x, y, w, h);
	}

	public void fillRect(float x, float y, float w, float h) {
		if (pathIsOpen) {
			super.beginPath();
		}
		super.fillRect(x, y, w, h);
	}

	public void strokeRect(float x, float y, float w, float h) {
		if (pathIsOpen) {
			super.beginPath();
		}
		super.strokeRect(x, y, w, h);
	}
}
